/*
 * lws-minimal-ws-server
 *
 * Written in 2010-2019 by Andy Green <andy@warmcat.com>
 *
 * This file is made available under the Creative Commons CC0 1.0
 * Universal Public Domain Dedication.
 *
 * This demonstrates the most minimal http server you can make with lws,
 * with an added websocket chat server.
 *
 * To keep it simple, it serves stuff in the subdirectory "./mount-origin" of
 * the directory it was started in.
 * You can change that by changing mount.origin.
 */

#include <libwebsockets.h>
#include <string.h>
#include <signal.h>

#include <event.h>

#include <event2/event.h>
#include <event2/http.h>
#include <event2/listener.h>
#include <event2/buffer.h>
#include <event2/util.h>
#include <event2/http_struct.h>
#include <event2/keyvalq_struct.h>
#include <event2/bufferevent.h>
#include <event2/bufferevent_compat.h>
#include <event2/http_compat.h>

//#include "cjson/cJSON.h"

#define HTTP_BUF 1024
volatile int  g_force_exit  = 0;

#define LWS_PLUGIN_STATIC
#include "protocol_lws_minimal.c"


static struct lws_protocols protocols[] = {
	{ "http", lws_callback_http_dummy, 0, 0, 0, NULL, 0},
	LWS_PLUGIN_PROTOCOL_MINIMAL,
	LWS_PROTOCOL_LIST_TERM
};

static const lws_retry_bo_t retry = {
	.secs_since_valid_ping = 3,
	.secs_since_valid_hangup = 10,
};

static int interrupted;

static const struct lws_http_mount mount = {
	/* .mount_next */		NULL,		/* linked-list "next" */
	/* .mountpoint */		"/",		/* mountpoint URL */
	/* .origin */			"./mount-origin",  /* serve from dir */
	/* .def */			"index.html",	/* default filename */
	/* .protocol */			NULL,
	/* .cgienv */			NULL,
	/* .extra_mimetypes */		NULL,
	/* .interpret */		NULL,
	/* .cgi_timeout */		0,
	/* .cache_max_age */		0,
	/* .auth_mask */		0,
	/* .cache_reusable */		0,
	/* .cache_revalidate */		0,
	/* .cache_intermediaries */	0,
	/* .origin_protocol */		LWSMPRO_FILE,	/* files in a dir */
	/* .mountpoint_len */		1,		/* char count */
	/* .basic_auth_login_file */	NULL,
};

void sigint_handler(int sig)
{
	interrupted = 1;
}

int ws_server(int argc, const char **argv)
{
	struct lws_context_creation_info info;
	struct lws_context *context;
	const char *p;
	int n = 0, logs = LLL_USER | LLL_ERR | LLL_WARN | LLL_NOTICE
			/* for LLL_ verbosity above NOTICE to be built into lws,
			 * lws must have been configured and built with
			 * -DCMAKE_BUILD_TYPE=DEBUG instead of =RELEASE */
			/* | LLL_INFO */ /* | LLL_PARSER */ /* | LLL_HEADER */
			/* | LLL_EXT */ /* | LLL_CLIENT */ /* | LLL_LATENCY */
			/* | LLL_DEBUG */;

//	signal(SIGINT, sigint_handler);

	if ((p = lws_cmdline_option(argc, argv, "-d")))
		logs = atoi(p);
//    logs = 255;
	lws_set_log_level(logs, NULL);
	lwsl_user("LWS minimal ws server | visit http://localhost:7681 (-s = use TLS / https)\n");

	memset(&info, 0, sizeof info); /* otherwise uninitialized garbage */
	info.port = 7681;
	info.mounts = &mount;
	info.protocols = protocols;
	info.vhost_name = "localhost";
	info.options =
		LWS_SERVER_OPTION_HTTP_HEADERS_SECURITY_BEST_PRACTICES_ENFORCE;

#if defined(LWS_WITH_TLS)
	if (lws_cmdline_option(argc, argv, "-s")) {
		lwsl_user("Server using TLS\n");
		info.options |= LWS_SERVER_OPTION_DO_SSL_GLOBAL_INIT;
		info.ssl_cert_filepath = "localhost-100y.cert";
		info.ssl_private_key_filepath = "localhost-100y.key";
	}
#endif

	if (lws_cmdline_option(argc, argv, "-h"))
		info.options |= LWS_SERVER_OPTION_VHOST_UPG_STRICT_HOST_CHECK;

	if (lws_cmdline_option(argc, argv, "-v"))
		info.retry_and_idle_policy = &retry;

	context = lws_create_context(&info);
	if (!context) {
		lwsl_err("lws init failed\n");
		return 1;
	}

	while (n >= 0 && !interrupted)
		n = lws_service(context, 0);

	lws_context_destroy(context);

	return 0;
}

int ws_thread(int argc, const char **argv)
{
  pthread_t tids;
  int ret = pthread_create(&tids, NULL, ws_server, NULL);
  printf("pthread_create error: error_code= %d \n", ret);
}

//curl -X POST  --data "{\"ueip_s\": [{ \"ue_ip\":\"192.169.0.3\"},{ \"ue_ip\":\"192.169.0.4\"},{ \"ue_ip\":\"192.169.0.5\"}]}"  http://192.168.2.142:8081/upf/v1/subscriptions/get_traffic_by_ueip  
static void
get_traffic_by_ueip_cb(struct evhttp_request *req, void *arg)
{
	printf("begin...\n");

	//struct evkeyvalq *headers;
	//struct evkeyval *header;
	struct evbuffer *buf;
	struct evbuffer *retbuff = NULL;

	//cJSON * reqRoot = NULL;
	char jsonStr[HTTP_BUF] = {0};

#if 0
	if(EVHTTP_REQ_POST != evhttp_request_get_command(req))
	{
		clib_upf_debug(um_http->log_fd,"http method not post\n");
		return ;
	}
	headers = evhttp_request_get_input_headers(req);
	for (header = headers->tqh_first; header; header = header->next.tqe_next) 
	{
		clib_upf_debug(um_http->log_fd, "%s:%s\n", header->key, header->value);
	}
#endif

	buf = evhttp_request_get_input_buffer(req);
	while (evbuffer_get_length(buf)) 
	{
		int n;
		n = evbuffer_remove(buf, jsonStr, sizeof(jsonStr));
		if (n <= 0)
			printf(" req data len is 0");
		//(void) fwrite(cbuf, 1, n, stdout);

	}

#if 0
	reqRoot = cJSON_Parse(jsonStr);     
	if (!reqRoot) 
	{
		printf("Error before: [%s]\n",cJSON_GetErrorPtr());
	}
	else
	{
		printf("req:%s\n\n", cJSON_PrintUnformatted(reqRoot));
	}
#endif

//	cJSON_Delete(reqRoot);

	//char *resp_Data  = "recv data\n";  

	retbuff = evbuffer_new();
	if(!retbuff)
	{
		printf("%s\n","retbuff is NULL.");			
		goto ERROR;
	}

	evbuffer_add(retbuff, jsonStr, strlen(jsonStr));
	evhttp_send_reply(req, 200, "OK", retbuff);

	printf("send http 200 ok  len:%d, jsonstr:%s\n", strlen(jsonStr), jsonStr);
        
	update_client(strlen(jsonStr), jsonStr);
	//websocket_client_send_data(jsonStr, strlen(jsonStr));

	printf("end...\n");

ERROR:
	if(retbuff)
		evbuffer_free(retbuff);
}

void *http_server_thread_fn(void *arg)
{
	struct event_base *base    = NULL;
	struct evhttp *http_server = NULL;

    char *http_addr = "0.0.0.0";
	int http_port = 8081;

	/* Initialize the event library */
	base        = event_base_new();
       
	http_server = evhttp_new(base);
	if(!http_server )
	{
		printf("%s:%d http server start failed\n",http_addr, http_port);
		return NULL;
	}
	evhttp_bind_socket(http_server, http_addr, http_port);
	evhttp_set_cb(http_server, "/upf/v1/subscriptions/get_traffic_by_ueip", get_traffic_by_ueip_cb, NULL);

	//Loop monitoring
	event_base_dispatch(base);

	if(http_server)
		evhttp_free(http_server);
	if (base)
		event_base_free(base);

	return NULL;
}

int  run_http_server_thread()
{ 
     pthread_t ntid;
     int err;
     err = pthread_create(&ntid, NULL, http_server_thread_fn, NULL);
     if(err != 0)
     {
         printf("can't create thread: %s",strerror(err));
         return -1;
     }

     //pthread_join(ntid, NULL);

     return  0;
}

int http_service()
{

    printf("http_service ..................\n");
    run_http_server_thread();
    return  0;    
}
